#include "Boulder.h"

using namespace std;

Image *Boulder::TextureImage;

Boulder::Boulder(float posx, float posy, float posz) {
   mEnabled = true;
   mCenter[X] = mPos[X] = mStartPos[X] = posx;
   mCenter[Y] = mPos[Y] = mStartPos[Y] = posy;
   mCenter[Z] = mPos[Z] = mStartPos[Z] = posz;

   mVel[X] = -3.0f;
   mVel[Y] = 0.0f;
   mVel[Z] = 0.0f;

   mState = 0;
   mAngle = 0.0f;   
   mQuad = gluNewQuadric();
   gluQuadricNormals(mQuad, GLU_SMOOTH);
   gluQuadricTexture(mQuad, GL_TRUE);

   mBoulderMaterial.mAmbient[0] = 0.5f;
   mBoulderMaterial.mAmbient[1] = 0.5f;
   mBoulderMaterial.mAmbient[2] = 0.0f;
   mBoulderMaterial.mAmbient[3] = 1.0f;

   mBoulderMaterial.mDiffuse[0] = 0.6f;
   mBoulderMaterial.mDiffuse[1] = 0.6f;
   mBoulderMaterial.mDiffuse[2] = 0.0f;
   mBoulderMaterial.mDiffuse[3] = 1.0f;

   mBoulderMaterial.mSpecular[0] = 0.5f;
   mBoulderMaterial.mSpecular[1] = 0.5f;
   mBoulderMaterial.mSpecular[2] = 0.5f;
   mBoulderMaterial.mSpecular[3] = 1.0f;

   mBoulderMaterial.mShininess[0] = 25.0f;
}

Boulder::~Boulder() {
   gluDeleteQuadric(mQuad);
}

GLvoid Boulder::LoadTexture(char* image_file, int texID) {	
 
    TextureImage = (Image *) malloc(sizeof(Image));
    if (TextureImage == NULL) {
        printf("Error allocating space for image");
        exit(1);
    }
    cout << "trying to load " << image_file << endl;
    if (!ImageLoad(image_file, TextureImage)) {
        exit(1);
    }  
        /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
    /*  x size from image, y size from image,                                              */    
    /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
    glBindTexture(GL_TEXTURE_2D, texID);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 TextureImage->sizeX, TextureImage->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
    //free( TextureImage );
}


/* BMP file loader */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
static unsigned int getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int Boulder::ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0;	
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}

//--------------------------------------------------------------------


void Boulder::setMaterial() {
   glMaterialfv(GL_FRONT, GL_AMBIENT,   mBoulderMaterial.mAmbient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   mBoulderMaterial.mDiffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR,  mBoulderMaterial.mSpecular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mBoulderMaterial.mShininess);   
}

void Boulder::reset() {
   mPos[X] = mCenter[X];
   mPos[Y] = mCenter[Y];
   mPos[Z] = mCenter[Z];
   mVel[Y] = 0.0f;
   mEnabled = true;
}

void Boulder::draw() {
   //drawBoundingBox();
   glPushMatrix();
      glColor3f(1.0f, 1.0f, 1.0f);  // Added White color
      //glEnable(GL_LIGHTING);
      glEnable(GL_TEXTURE_2D);
      glTranslatef(mPos[X], mPos[Y], mPos[Z]);
      glRotatef(mAngle, 0.0f, 0.0f, 1.0f);
      glBindTexture(GL_TEXTURE_2D, BOULDERTEX);
      gluSphere(mQuad, 1.0, 32, 32);
      glDisable(GL_TEXTURE_2D);
      //glDisable(GL_LIGHTING);
   glPopMatrix();
}

void Boulder::reinitialize() {}

void Boulder::step(double dt, Scene *scene) {
   if (mEnabled) {
      double dx = mVel[X] * dt;
      double dy = mVel[Y] * dt;
      double dz = mVel[Z] * dt;

      mAngle += dt * 75.0;
      if (mAngle > TWOPI)
         mAngle -= TWOPI;

      this->collide(&scene->mObjects, &dx, &dy, &dz);
      if (scene->mPrevScene)
         this->collide(&scene->mPrevScene->mObjects, &dx, &dy, &dz);   
      if (scene->mNextScene)
         this->collide(&scene->mNextScene->mObjects, &dx, &dy, &dz);

      // This line of code should not be necessary, but I'm having no luck
      // without it. Sets the downward velocity to zero when on the ground.
      if (dy == 0.0f && mVel[Y] != 0.0f)
         mVel[Y] = 0.0f;

      mVel[Y] += -9.8 * dt;

      mPos[X] += dx;
      mPos[Y] += dy;

      mMin[X] = mPos[X] - 1.0;
      mMin[Y] = mPos[Y] - 1.0;
      mMin[Z] = mPos[Z] - 1.0;

      mMax[X] = mPos[X] + 1.0;
      mMax[Y] = mPos[Y] + 1.0;
      mMax[Z] = mPos[Z] + 1.0;

      mEnabled = mPos[Y] > -5.0f;
   }

   if (!mEnabled)
      reset();
}

void Boulder::collide(std::vector<GameObject *> *objs, double *dx, double *dy, double *dz) {
   for (std::vector<GameObject *>::iterator itr = objs->begin(); itr != objs->end(); itr++) {
      (*itr)->collide(this, dx, dy, dz);
   }
}

void Boulder::collide(Player *player, double *dx, double *dy, double *dz) {
   if (mEnabled && collideAxis(player, X) && collideAxis(player, Y) &&
    collideAxis(player, Z)) {
      player->mLife = -1;
   }
}
