#ifndef _CHARACTER_H_
#define _CHARACTER_H_

//#include "Scene.h"
#include "Constants.h"

class Scene;
class Player;

class Character {
friend class Stage1;
friend class Stage2;
friend class MovingSpike;
friend class Cave;
friend class GameObject;
friend class Coin;
friend class Ground;
friend class Platform;
friend class Boulder;
friend class HorizontalPlatform;
friend class FallingPlatform;
friend class SwingingPlatform;
friend class ParticleSystem;
friend class FlowSystem;
friend class FloatingEnemy;
friend class WalkingEnemy;
friend class PacingEnemy;
friend class Inset;
friend class CrushingWall;

protected:
   char mState;              /* Various flags for different character states */
   bool mEnabled;
   double mInvisTime;
   unsigned char mCoins;     /* Number of coins, only applicable for Player */
   double mStartPos[3], mThick[3];
   double mMin[3], mMax[3];   /* Bounding Box Variables */
   double mPos[3], mVel[3];   /* Position and Velocity */
   double mAcc[3];
   float mLife;              /* Hit Points, only applicable for Player and Bosses(?) */

   void drawBoundingBox();

   inline bool collideAxis(Character *character, int axis) {
      return character->mMax[axis] > mMin[axis] &&
       character->mMin[axis] < mMax[axis];
   }

   inline bool collideAxis(Character *character, int axis, double *offset) {
      return character->mMax[axis] + *offset > mMin[axis] &&
       character->mMin[axis] + *offset < mMax[axis];
   }

public:
   Character() {}
   virtual ~Character() {}

   virtual void collide(Player *player, double *dx, double *dy, double *dz) = 0;
   virtual void decrLife() = 0;
   virtual void draw() = 0;
   virtual void reinitialize() = 0;
   virtual void step(double, Scene *) = 0;
   inline void incrCoins() {mCoins++;}
};

#endif