#include "Coin.h"

float Coin::mAngle;

Coin::Coin(float posx, float posy, float posz) {
   mEnabled = true;
   mQuad = gluNewQuadric();
   gluQuadricNormals(mQuad, GLU_SMOOTH);
   gluQuadricTexture(mQuad, GL_TRUE);

   mMin[X] = posx - 0.25f;
   mMin[Y] = posy - 0.25f;
   mMin[Z] = posz - 0.25f;
   
   mMax[X] = posx + 0.25f;
   mMax[Y] = posy + 0.25f;
   mMax[Z] = posz + 0.25f;
   
   mAngle = 0.0f;

   mCoinMaterial.mAmbient[0] = 0.5f;
   mCoinMaterial.mAmbient[1] = 0.5f;
   mCoinMaterial.mAmbient[2] = 0.0f;
   mCoinMaterial.mAmbient[3] = 1.0f;

   mCoinMaterial.mDiffuse[0] = 0.6f;
   mCoinMaterial.mDiffuse[1] = 0.6f;
   mCoinMaterial.mDiffuse[2] = 0.0f;
   mCoinMaterial.mDiffuse[3] = 1.0f;

   mCoinMaterial.mSpecular[0] = 0.5f;
   mCoinMaterial.mSpecular[1] = 0.5f;
   mCoinMaterial.mSpecular[2] = 0.5f;
   mCoinMaterial.mSpecular[3] = 1.0f;

   mCoinMaterial.mShininess[0] = 25.0f;
}

Coin::~Coin() {
   gluDeleteQuadric(mQuad);   
}

void Coin::setMaterial() {
   glMaterialfv(GL_FRONT, GL_AMBIENT,   mCoinMaterial.mAmbient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   mCoinMaterial.mDiffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR,  mCoinMaterial.mSpecular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mCoinMaterial.mShininess);   
}

void Coin::collide(Character *character, double *dx, double *dy, double *dz)  {
   if (mEnabled && character->mState & PLAYER && collideAxis(character, X) && 
    collideAxis(character, Y) && collideAxis(character, Z)) {
      #if defined(WIN32)
      PlaySound("coin.wav", NULL, SND_FILENAME | SND_ASYNC);
      #endif

      character->incrCoins();
      mEnabled = false;
   }
}

void Coin::draw() {
   if (mEnabled) {
      glColor3f(1.0f, 1.0f, 0.0f);
      //drawBoundingBox();

      glPushMatrix();
         glEnable(GL_LIGHTING);
         setMaterial();

         glTranslatef(mMin[X] + 0.25f, mMin[Y] + 0.25f, mMin[Z] + 0.25f);
         glRotatef(mAngle, 0.0f, 1.0f, 0.0f);
         gluCylinder(mQuad, 0.25f, 0.25f, 0.1f, 32, 32);   

         glTranslatef(0.0f, 0.0f, 0.1f);
         gluDisk(mQuad, 0.0f, 0.25f, 32, 32);

         glRotatef(180, 0.0f, 1.0f, 0.0f);
         glTranslatef(0.0f, 0.0f, 0.1f);
         gluDisk(mQuad, 0.0f, 0.25f, 32, 32);

         glDisable(GL_LIGHTING);
      glPopMatrix();
   }
}

void Coin::reinitialize() {
   mEnabled = true;
}

void Coin::gstep(double dt) {
   mAngle = mAngle > 360.0f ? 0.0 : mAngle + 100.0f * dt;
}
