#include "CrushingWall.h"
#include "Player.h"
#include <assert.h>

Image *CrushingWall::TextureImage;

CrushingWall::CrushingWall(float pX, float pY, float pZ, float endPX, float endPY, float endPZ)
{
//   LoadTexture("carpet1abump.bmp", 3);

   currFrame = STARTFRAME;
   increment = 1.0f;

   mPos[X] = pX;
   mPos[Y] = pY;
   mPos[Z] = pZ;

   endPos[X] = endPX;
   endPos[Y] = endPY;
   endPos[Z] = endPZ;

   trans.x = 0.0f;
   trans.y = 0.0f;
   trans.z = 0.0f;

   mMin[X] = mPos[X] - 4.0f;
   mMin[Y] = mPos[Y] - 5.0f;
   mMin[Z] = mPos[Z] - 5.0f;

   mMax[X] = mPos[X] + 4.0f;
   mMax[Y] = mPos[Y] + 5.0f;
   mMax[Z] = mPos[Z] + 5.0f;
}

CrushingWall::~CrushingWall()
{

}

GLvoid CrushingWall::LoadTexture(char* image_file, int texID) {	
 
    TextureImage = (Image *) malloc(sizeof(Image));
    if (TextureImage == NULL) {
        printf("Error allocating space for image");
        exit(1);
    }
    cout << "trying to load " << image_file << endl;
    if (!ImageLoad(image_file, TextureImage)) {
        exit(1);
    }  
        /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
    /*  x size from image, y size from image,                                              */    
    /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
    glBindTexture(GL_TEXTURE_2D, texID);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 TextureImage->sizeX, TextureImage->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
    //free( TextureImage );
}


/* BMP file loader */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
static unsigned int getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int CrushingWall::ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0;	
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}

//--------------------------------------------------------------------

void CrushingWall::draw()
{
   glColor3f(1.0f, 1.0f, 1.0f);
   //drawBoundingBox();
   drawFaces(CARPETTEX, true, false);
}

void CrushingWall::linearMotion()
{
   float t = (currFrame - STARTFRAME) / (ENDFRAME - STARTFRAME);

   mMin[X] += t * (endPos[X] - mPos[X]) - trans.x;
   mMax[X] += t * (endPos[X] - mPos[X]) - trans.x;
   mMin[Y] += t * (endPos[Y] - mPos[Y]) - trans.y;
   mMax[Y] += t * (endPos[Y] - mPos[Y]) - trans.y;
   mMin[Z] += t * (endPos[Z] - mPos[Z]) - trans.z;
   mMax[Z] += t * (endPos[Z] - mPos[Z]) - trans.z;

   trans.x =  t*(endPos[X] - mPos[X]);
   trans.y =  t*(endPos[Y] - mPos[Y]);
   trans.z =  t*(endPos[Z] - mPos[Z]);
}


void CrushingWall::step(double dt)
{
   linearMotion();
   currFrame += increment;

   /* keeps t between [0, 1] */
   if(currFrame == ENDFRAME)
      increment = -0.5f;
   if(currFrame == STARTFRAME)
      increment = 0.5f;
}

void CrushingWall::reinitialize()
{
   trans.x = 0.0f;
   trans.y = 0.0f;
   trans.z = 0.0f;
}

// Collide like boulder collision
void CrushingWall::collide(Character *character, double *dx, double *dy, double *dz)
{
      if (collideAxis(character, X) && collideAxis(character, Y) && collideAxis(character, Z))
      {
         character->mLife = -1;
      }
}