#include "FlowSystem.h"

float FlowSystem::Colors[9][3] = {
   {1.0f, 1.0f, 1.0f}, {1.0f, 0.9f, 0.0f}, {1.0f, 0.6f, 0.0f},
   {1.0f, 0.3f, 0.0f}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0},
   {1.0, 0.0, 0.0}, {0.9, 0.0, 0.0}, {0.9, 0.0, 0.0}
};

unsigned int FlowSystem::texture[1];

FlowSystem::FlowSystem(unsigned int tex) {
   texture[0] = tex;
}

FlowSystem::FlowSystem(float minx, float miny, float minz, float maxx, float maxy, float maxz, unsigned int tex) {
   mMin[X] = minx;
   mMin[Y] = miny;
   mMin[Z] = minz;
   
   mMax[X] = maxx;
   mMax[Y] = maxy;
   mMax[Z] = maxz;

   mTime = 0.0f;
   mEnabled = false;
   texture[0] = tex;
}

FlowSystem::~FlowSystem() {

}

void FlowSystem::collide(Character *character, double *dx, double *dy, double *dz) {
   if (mEnabled && collideAxis(character, X) && collideAxis(character, Y) && collideAxis(character, Z)) {
      character->decrLife();
   }
}

void FlowSystem::draw() {
   glColor3f(0.0, 1.0, 0.0);
   //drawBoundingBox();
//if (mEnabled) {

   glPushMatrix();
      glDisable(GL_DEPTH_TEST);						// Disables Depth Testing
   	glEnable(GL_BLEND);							// Enable Blending
   	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					// Type Of Blending To Perform
   	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);			// Really Nice Perspective Calculations
   	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);					// Really Nice Point Smoothing

      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, FlowSystem::texture[0]);

      for (int ndx = 0; ndx < MAX_FLOW; ndx++) {
         if (mParticles[ndx].mActive) {
//            float dist = sqrt(pow(mParticles[ndx].mPositionX, 2) + pow(mParticles[ndx].mPositionY, 2) + pow(mParticles[ndx].mPositionZ, 2));

            float dist = sqrt(pow(mParticles[ndx].mPositionX - (mMax[X] + mMin[X]) / 2, 2) +
             pow(mParticles[ndx].mPositionY - (mMax[Y] + mMin[Y]) / 2, 2) +
             pow(mParticles[ndx].mPositionZ - (mMax[Z] + mMin[Z]) / 2, 2));

            dist *= 8;

            glColor4f(Colors[(int)dist % 9][0], Colors[(int)dist % 9][1], Colors[(int)dist % 9][2], mParticles[ndx].mLife);

            glBegin(GL_TRIANGLE_STRIP);

               glTexCoord2d(1, 1);
               glVertex3f(mParticles[ndx].mPositionX + 0.1f, mParticles[ndx].mPositionY +
                0.1f, mParticles[ndx].mPositionZ);

               glTexCoord2d(0, 1);
               glVertex3f(mParticles[ndx].mPositionX - 0.1f, mParticles[ndx].mPositionY +
                0.1f, mParticles[ndx].mPositionZ);

               glTexCoord2d(1, 0);
               glVertex3f(mParticles[ndx].mPositionX + 0.1f, mParticles[ndx].mPositionY -
                0.1f, mParticles[ndx].mPositionZ);

               glTexCoord2d(0, 0);
               glVertex3f(mParticles[ndx].mPositionX - 0.1f, mParticles[ndx].mPositionY -
                0.1f, mParticles[ndx].mPositionZ);
            glEnd();

            glBegin(GL_TRIANGLE_STRIP);
               glTexCoord2d(1, 1);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY + 0.05f,
                mParticles[ndx].mPositionZ + 0.05f);

               glTexCoord2d(0, 1);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY + 0.05f, 
                mParticles[ndx].mPositionZ - 0.05f);

               glTexCoord2d(1, 0);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY - 0.05f, 
                mParticles[ndx].mPositionZ + 0.05f);

               glTexCoord2d(0, 0);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY - 0.05f,
                mParticles[ndx].mPositionZ - 0.05f);
            glEnd();

         }
      }

      glDisable(GL_TEXTURE_2D);
   	glDisable(GL_BLEND);							// Enable Blending
      glEnable(GL_DEPTH_TEST);						// Disables Depth Testing
   glPopMatrix();
//   }
}

void FlowSystem::reinitialize() {}

void FlowSystem::step(double dt) {
   mTime += dt;

   if (mTime > 2.0f) {
      mTime = 0.0f;
      mEnabled = !mEnabled;

      if (mEnabled) {
         #if defined(WIN32)
         PlaySound("thunder.wav", NULL, SND_FILENAME | SND_ASYNC);
         #endif
      }
   }

   dt *= 20.0f;
   
   for (int ndx = 0; ndx < MAX_FLOW; ndx++) {
      if (mParticles[ndx].mLife > 0.0f) {
         mParticles[ndx].mPositionX += mParticles[ndx].mDirectionX * dt * 0.001f;
         mParticles[ndx].mPositionY += mParticles[ndx].mDirectionY * dt * 0.001f;
         mParticles[ndx].mPositionZ += mParticles[ndx].mDirectionZ * dt * 0.001f;

         mParticles[ndx].mDirectionX += mParticles[ndx].mAccelX * dt;
         mParticles[ndx].mDirectionY += mParticles[ndx].mAccelY * dt;
         mParticles[ndx].mDirectionZ += mParticles[ndx].mAccelZ * dt;

   //      mParticles[ndx].mPositionY += 0.01 * pow(1.5f, -(abs(mParticles[ndx].mPositionX))) * dt;

         mParticles[ndx].mLife -= mParticles[ndx].mFade * dt;
      }
     
      mParticles[ndx].mActive = mParticles[ndx].mLife > 0.0f;

      if (mEnabled && mParticles[ndx].mLife <= 0.0f) {
         mParticles[ndx].flowInit();

         mParticles[ndx].setPosition(
            (mMax[X] + mMin[X]) / 2.0f, 
            (mMax[Y] + mMin[Y]) / 2.0f, 
            (mMax[Z] + mMin[Z]) / 2.0f - 0.5f);

//         ((float)rand() / (float)RAND_MAX) * (mMax[X] - mMin[X]) - (mMax[X] - mMin[X]) / 2.0f + (mMax[X] + mMin[X]) / 2.0f,
//         ((float)rand() / (float)RAND_MAX) * (mMax[Y] - mMin[Y]) - (mMax[Y] - mMin[Y]) / 2.0f + (mMax[Y] + mMin[Y]) / 2.0f,
//         ((float)rand() / (float)RAND_MAX) * (mMax[Z] - mMin[Z]) - (mMax[Z] - mMin[Z]) / 2.0f + (mMax[Z] + mMin[Z]) / 2.0f);
//            ((float)rand() / (float)RAND_MAX) * 0f - 0.5,
//            ((float)rand() / (float)RAND_MAX) * 0f - 0.1,
//            0.0f, 0.0f,
//            0.0f);
      }
   }
}