#include "HorizontalPlatform.h"
#include <math.h>

Image *HorizontalPlatform::TextureImage;

HorizontalPlatform::HorizontalPlatform(float centerx, float centery, float centerz,
                      float width, float height, float depth, float distance, char axis) {
   mEndPos[X] = centerx;
   mEndPos[Y] = centery;
   mEndPos[Z] = centerz;

   mThick[X] = width;
   mThick[Y] = height;
   mThick[Z] = depth;
   
   mDistance = distance;
   mAxis = axis;
   mTime = 0.0f;

   mMin[X] = centerx - width * 0.5f;
   mMax[X] = mMin[X] + width;
   mMin[Y] = centery - height * 0.5f;
   mMax[Y] = mMin[Y] + height;
   mMin[Z] = centerz - depth * 0.5f;
   mMax[Z] = mMin[Z] + depth;
}

HorizontalPlatform::~HorizontalPlatform() {}

GLvoid HorizontalPlatform::LoadTexture(char* image_file, int texID) {	
 
    TextureImage = (Image *) malloc(sizeof(Image));
    if (TextureImage == NULL) {
        printf("Error allocating space for image");
        exit(1);
    }
    cout << "trying to load " << image_file << endl;
    if (!ImageLoad(image_file, TextureImage)) {
        exit(1);
    }  
        /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
    /*  x size from image, y size from image,                                              */    
    /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
    glBindTexture(GL_TEXTURE_2D, texID);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 TextureImage->sizeX, TextureImage->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
    //free( TextureImage );
}


/* BMP file loader */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
static unsigned int getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

static unsigned int getshort(FILE *fp){
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int HorizontalPlatform::ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0;	
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}

//--------------------------------------------------------------------

void HorizontalPlatform::collide(Character *character, double *dx, double *dy, double *dz) {
   if (collideAxis(character, X) && collideAxis(character, Y)) { // player colliding BEFORE movement, ie platform moved up
      character->mState &= ~FALLING;
      character->mPos[Y] += mMax[Y] - character->mMin[Y];
      character->mMax[Y] += mMax[Y] - character->mMin[Y];
      character->mMin[Y] += mMax[Y] - character->mMin[Y];
      *dy += mMax[Y] - character->mMin[Y];
   }
   else if (mAxis == Y && collideAxis(character, X) && 
    character->mMin[Y] > mMax[Y] && character->mMin[Y] - mMax[Y] < 0.10f * 
    mDistance && !(character->mState & JUMPING)) {
      character->mState &= ~FALLING;
      *dy -= character->mMin[Y] - mMax[Y];
      character->mVel[Y] = 0;
   }

   // if dx causes collsion
   else if (collideAxis(character, X, dx) && collideAxis(character, Y)) {
      character->mVel[X] = 0;
      *dx = character->mPos[X] > mMax[X] ? mMax[X] - character->mMin[X] + 0.01f :
       mMin[X] - character->mMax[X] - 0.01f;         
   }


   if (collideAxis(character, X) && collideAxis(character, Y, dy)) {
      // intersection with y                
      if (character->mPos[Y] > mMax[Y]) {             // character is above platform
         character->mState &= ~FALLING;               // character has landed from a jump
         *dy = mMax[Y] - character->mMin[Y];
         character->mVel[Y] = 0;

         if (character->mState & PLAYER) {
            if (mPlayerPrev > -10.0f) {
               if (mAxis == X) {
                  *dx += mPos[X] - mPlayerPrev;
               }
            }
            mPlayerPrev = mPos[mAxis];
         }
      }
      else {                                          // character collided with bottom of platform
         mPlayerPrev = -100.0f;

         if (character->mVel[Y] > 0) {                // if velocity is positive, stop going up
            *dy = mMin[Y] - character->mMax[Y] - 0.01f; 
            character->mVel[Y] = 0;
         }
      }
      
   }
   else if (character->mState & PLAYER) {
      mPlayerPrev = -100.0f; 
   }
}

void HorizontalPlatform::draw() {
   glColor3f(0.0, 1.0, 0.0);
   //drawBoundingBox();
   drawFaces(HORZNTPLFM);
}

void HorizontalPlatform::reinitialize() {}

void HorizontalPlatform::step(double dt) {
   mTime += dt;

   if (mTime > TWOPI)
      mTime -= TWOPI;

   mPos[mAxis] = -1 * sin(mTime) * mDistance + mEndPos[mAxis];

   mMin[mAxis] = mPos[mAxis] - mThick[mAxis] * 0.5f;
   mMax[mAxis] = mMin[mAxis] + mThick[mAxis];
}