#include "ParticleSystem.h"

float ParticleSystem::Colors[9][3] = {
   {1.0f, 1.0f, 1.0f}, {1.0f, 0.9f, 0.0f}, {1.0f, 0.6f, 0.0f},
   {1.0f, 0.3f, 0.0f}, {1.0, 0.0, 0.0}, {1.0, 0.0, 0.0},
   {1.0, 0.0, 0.0}, {0.9, 0.0, 0.0}, {0.9, 0.0, 0.0}
};

unsigned int ParticleSystem::texture[1];

ParticleSystem::ParticleSystem(float minx, float miny, float minz,
                               float maxx, float maxy, float maxz, float size) {

   mMin[X] = minx;
   mMin[Y] = miny;
   mMin[Z] = minz;

   mMax[X] = maxx;
   mMax[Y] = maxy;
   mMax[Z] = maxz;
   
   mSize = size;
   mParticles = new Particle[size];

   for (int ndx = 0; ndx < mSize; ndx++) {
      mParticles[ndx].init();
      mParticles[ndx].setPosition(
         ((float)rand() / (float)RAND_MAX) * (mMax[X] - mMin[X]) - (mMax[X] - mMin[X]) * 0.5f + (mMax[X] + mMin[X]) * 0.5f,
         ((float)rand() / (float)RAND_MAX) * (mMax[Y] - mMin[Y]) - (mMax[Y] - mMin[Y]) * 0.5f + (mMax[Y] + mMin[Y]) * 0.5f,
         ((float)rand() / (float)RAND_MAX) * (mMax[Z] - mMin[Z]) - (mMax[Z] - mMin[Z]) * 0.5f + (mMax[Z] + mMin[Z]) * 0.5f);
   }
}

ParticleSystem::~ParticleSystem() {
   delete [] mParticles;
}
/*
AUX_RGBImageRec *ParticleSystem::LoadBMP(char *fname) {
   FILE *file = NULL;

   printf("Opening %s\n", fname);
   if (!fname)
      return NULL;

   file = fopen(fname, "r");
   if (file) {
      printf("File opened\n");
      fclose(file);
      return auxDIBImageLoad(fname);
   }

   printf("failed to open file\n");
   return NULL;
}
*/
/* Reads in a .RAW file */
bool ParticleSystem::setup_textures() {
   glGenTextures(1, &texture[0]);
   RGBIMG img;
   FILE *file;
   long fsize;
   
   img.w = 32;
   img.h = 32;
   img.data = new GLubyte[(((3 * 32 + 3) >> 2) << 2) * 32];
   
   file = fopen("particle_32x32.raw", "rb");
   if (!file) return false;
   GLubyte *p;

   fseek(file, 0L, SEEK_END);
   fsize = ftell(file);
   
   fseek(file, 0L, SEEK_SET);
   p = img.data;
   while (fsize > 0) {
      fread(p, 1, 1, file);
      p++;
      fsize--;
   }
   fclose(file);
   
   glBindTexture(GL_TEXTURE_2D, texture[0]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, img.w, img.h, 0, GL_RGB, GL_UNSIGNED_BYTE, img.data);
   delete img.data;
   return true;   
}
/*
int ParticleSystem::LoadGLTextures() {
   int status = false;
   AUX_RGBImageRec *texImage[1];
   memset(texImage, 0, sizeof(void *) * 1);

   if (texImage[0] = LoadBMP("Particle.bmp")) {
      status = true;
      glGenTextures(1, &texture[0]);
      glBindTexture(GL_TEXTURE_2D, texture[0]);

      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

      glTexImage2D(GL_TEXTURE_2D, 0, 3, texImage[0]->sizeX, texImage[0]->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, texImage[0]->data);
   }

   if (texImage[0]) {
      if (texImage[0]->data)
         free(texImage[0]->data);
      free(texImage[0]);
   }

   return status;
}
*/

void ParticleSystem::collide(Character *character, double *dx, double *dy, double *dz) {
   if (collideAxis(character, X) && collideAxis(character, Y) && 
    collideAxis(character, Z)) {
      character->decrLife();
   }
}

void ParticleSystem::draw() {
   glColor3f(0.0, 1.0, 0.0);
   //drawBoundingBox();

   glPushMatrix();
      glDisable(GL_DEPTH_TEST);					                     // Disables Depth Testing
      glDisable(GL_LIGHTING);                                     // Disable Lighting
   	glEnable(GL_BLEND);							                     // Enable Blending
   	glBlendFunc(GL_SRC_ALPHA,GL_ONE);					            // Type Of Blending To Perform
   	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);		   	// Really Nice Perspective Calculations
   	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				      	// Really Nice Point Smoothing

      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, ParticleSystem::texture[0]);

      for (int ndx = 0; ndx < mSize; ndx++) {
         if (mParticles[ndx].mActive) {
//            float dist = sqrt(pow(mParticles[ndx].mPositionX - (mMax[X] + mMin[X]) / 2, 2) +
//             pow(mParticles[ndx].mPositionY - (mMax[Y] + mMin[Y]) / 2, 2) +
//             pow(mParticles[ndx].mPositionZ - (mMax[Z] + mMin[Z]) / 2, 2));

            float dist = sqrt(pow(mParticles[ndx].mPositionX - (mMax[X] + mMin[X]) / 2, 2) +
             pow(mParticles[ndx].mPositionY - (mMin[Y]), 2) +
             pow(mParticles[ndx].mPositionZ - (mMax[Z] + mMin[Z]) / 2, 2));


            dist *= 8;
            glColor4f(Colors[(int)dist % 9][0], Colors[(int)dist % 9][1], 
             Colors[(int)dist % 9][2], mParticles[ndx].mLife);

            glBegin(GL_TRIANGLE_STRIP);
               glTexCoord2d(1, 1);
               glVertex3f(mParticles[ndx].mPositionX + 0.05f, 
                mParticles[ndx].mPositionY + 0.05f, mParticles[ndx].mPositionZ);

               glTexCoord2d(0, 1);
               glVertex3f(mParticles[ndx].mPositionX - 0.05f, 
                mParticles[ndx].mPositionY + 0.05f, mParticles[ndx].mPositionZ);

               glTexCoord2d(1, 0);
               glVertex3f(mParticles[ndx].mPositionX + 0.05f, 
                mParticles[ndx].mPositionY - 0.05f, mParticles[ndx].mPositionZ);

               glTexCoord2d(0, 0);
               glVertex3f(mParticles[ndx].mPositionX - 0.05f, 
                mParticles[ndx].mPositionY - 0.05f, mParticles[ndx].mPositionZ);
            glEnd();

            glBegin(GL_TRIANGLE_STRIP);
               glTexCoord2d(1, 1);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY + 0.05f,
                mParticles[ndx].mPositionZ + 0.05f);

               glTexCoord2d(0, 1);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY + 0.05f, 
                mParticles[ndx].mPositionZ - 0.05f);

               glTexCoord2d(1, 0);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY - 0.05f, 
                mParticles[ndx].mPositionZ + 0.05f);

               glTexCoord2d(0, 0);
               glVertex3f(mParticles[ndx].mPositionX, mParticles[ndx].mPositionY - 0.05f,
                mParticles[ndx].mPositionZ - 0.05f);
            glEnd();
         }
      }

      glDisable(GL_BLEND);
      glDisable(GL_TEXTURE_2D);
      glEnable(GL_DEPTH_TEST);
   glPopMatrix();
}

void ParticleSystem::reinitialize() {}

void ParticleSystem::step(double dt) {
   dt *= 20;

   for (int ndx = 0; ndx < mSize; ndx++) {
      mParticles[ndx].mPositionX += mParticles[ndx].mDirectionX * 0.001f * dt;
      mParticles[ndx].mPositionY += mParticles[ndx].mDirectionY * 0.001f * dt;
      mParticles[ndx].mPositionZ += mParticles[ndx].mDirectionZ * 0.001f * dt;

      mParticles[ndx].mDirectionX += mParticles[ndx].mAccelX * dt * dt;
      mParticles[ndx].mDirectionY += mParticles[ndx].mAccelY * dt * dt;
      mParticles[ndx].mDirectionZ += mParticles[ndx].mAccelZ * dt * dt;

      mParticles[ndx].mPositionY += 0.01 * pow(1.5f, 
      mParticles[ndx].mPositionX <= 0 ? mParticles[ndx].mPositionX : -mParticles[ndx].mPositionX) * dt;

      mParticles[ndx].mLife -= mParticles[ndx].mFade * dt;

      if (mParticles[ndx].mLife <= 0.0f) {
         mParticles[ndx].init();

         mParticles[ndx].setPosition(
         ((float)rand() / (float)RAND_MAX) * (mMax[X] - mMin[X]) - (mMax[X] - mMin[X]) *
          0.5f + (mMax[X] + mMin[X]) * 0.5f,
         ((float)rand() / (float)RAND_MAX) * (mMax[Y] - mMin[Y]) - (mMax[Y] - mMin[Y]) * 
          0.5f + (mMax[Y] + mMin[Y]) * 0.5f,
         ((float)rand() / (float)RAND_MAX) * (mMax[Z] - mMin[Z]) - (mMax[Z] - mMin[Z]) * 
          0.5f + (mMax[Z] + mMin[Z]) * 0.5f);
      }
   }
}