#ifndef _PLATFORM_H_
#define _PLATFORM_H_

#include <iostream>
#include <cmath>

#include "Character.h"
#include "GameObject.h"
#include "Vector3.h"

//static const float STARTFRAME = 0.0f;
//static const float ENDFRAME = 100.0f;
//static const float RADIUS = 10.0f;
enum Action {NONE, PENDULUM, LINEAR, FASTSTOP, SLOWSTOP, SPIRAL};

using namespace std;

class Platform : public GameObject {
private:
   float currFrame;
   float increment;
   float angle;
   Action grndActn;  /* Each ground object can do different types of motions */
   Vector3 pos;      /* Objects' starting position */
   Vector3 endPos;   /* Objects' ending position */
   Vector3 trans;    /* Keeps track of transformations between interpolation */


public:
   Platform(float minx, float miny, float minz, float maxx, float maxy, float maxz, Action act = NONE) {
      mMin[X] = minx;
      mMin[Y] = miny;
      mMin[Z] = minz;

      mMax[X] = maxx;
      mMax[Y] = maxy;
      mMax[Z] = maxz;

      pos.x = (mMin[X] + mMax[X]) / 2.0f;
      pos.y = (mMin[Y] + mMax[Y]) / 2.0f;
      pos.z = (mMin[Z] + mMax[Z]) / 2.0f;

      endPos.x = pos.x;
      endPos.y = pos.y;
      endPos.z = pos.z;

      trans.x = 0.0f;
      trans.y = 0.0f;
      trans.z = 0.0f;

      currFrame = STARTFRAME;
      increment = 1.0f;
      grndActn = act;
      angle = 0.0f;
   };

   virtual ~Platform() {}

   void setEndPosition(Vector3 *v);
   void draw();
   void step(double dt);
   void collide(Character *, double *dx, double *dy, double *dz);
   void PendulumMotion(void);
   void linearMotion(void);
   void spiralMotion(void);
};

#endif