#include "Player.h"
#include "Scene.h"
#include <iostream>
using namespace std;

#define JUMPTIME   0.30
#define JUMPACCEL 13.0f
#define JUMPVEL    2.0f

Player::Player() {
   mCoins = 0;
   mState = PLAYER;
   mLife = 100.0f;
   mLives = 3;

   mPos[X] = mPos[Z] = 0;
   mPos[Y] = 10.0f;

   mMin[X] = mPos[X] - 0.5;
   mMin[Y] = mPos[Y] - 0.5;
   mMin[Z] = mPos[Z] - 0.5;

   mMax[X] = mPos[X] + 0.5;
   mMax[Y] = mPos[Y] + 0.5;
   mMax[Z] = mPos[Z] + 0.5;

   mVel[X] = mVel[Y] = mVel[Z] = 0;
   mAcc[X] = mAcc[Y] = mAcc[Z] = 0;

   mQuad = gluNewQuadric();
   gluQuadricNormals(mQuad, GLU_SMOOTH);
   gluQuadricTexture(mQuad, GL_TRUE);

   myModel = new CMD2Model;
   //myModel->Load("models\\ogro\\tris.md2", "models\\ogro\\ogrobase.pcx");
   myModel->Load("MARVIN\\tris.md2", "MARVIN\\Marvin.pcx");
   //myModel->Load("OGRO\\tris.MD2", "OGRO\\hobgoblin.bmp");
   modelState = MODEL_IDLE;
}

Player::~Player() {

}

void Player::jump() {
   if (!(mState & FALLING)) {                // PLAYER CAN NOT JUMP WHILE FALLING
      if (!(mState & JUMPING)) {             // IF PLAYER NOT JUMPING, START TIMER
         #if defined(WIN32)
         PlaySound("jump.wav", NULL, SND_FILENAME | SND_ASYNC);
         #endif

         mJumpTime = 0.0;
         mState |= JUMPING;
         mAcc[Y] = JUMPACCEL;
         mVel[Y] = JUMPVEL;

         myModel->SetState(MODEL_JUMP);
      }
      else if (mState & JUMPING && mJumpTime < JUMPTIME) {
         mAcc[Y] -= 0.1f;
//         mVel[Y] = 5.0f;         
      }
      else if (mState & JUMPING && mJumpTime >= JUMPTIME) {
         myModel->SetState(MODEL_IDLE);
         mAcc[Y] = 0.0f;
         mJumpTime = 0.0;
//         mVel[Y] = 1.0f;
         mState &= ~JUMPING;
         mState |= FALLING;
      }
   }
}

/* Forces the player to jump, even if the player is
   in the air */
void Player::forceJump() {
   #if defined(WIN32)
   PlaySound("bounce.wav", NULL, SND_FILENAME | SND_ASYNC);
   #endif

   mState &= ~FALLING;
   mState |= JUMPING;
   mJumpTime = 0.0f;
   mVel[Y] = JUMPVEL;
   mAcc[Y] = JUMPACCEL;
   myModel->SetState(MODEL_JUMP);
}

void Player::stopJump() {
   if (mState & JUMPING) {
      myModel->SetState(MODEL_IDLE);
      mState &= ~JUMPING;
      mState |= FALLING;
//      mVel[Y] = 1.0f;
      mAcc[Y] = 0.0f;
   }
}

void Player::draw() {

   if (mState & INVINCIBLE && 
      (mInvisTime > 0.1 && mInvisTime < 0.2 || mInvisTime > 0.3 && mInvisTime < 0.4 ||
      mInvisTime > 0.5 && mInvisTime < 0.6 || mInvisTime > 0.7)) {
         return;
   }

   if (mLife < 0.0f)
      glColor3f(1.0f, 0.0f, 0.0f);
   else if (mState & INVINCIBLE) {
      glColor3f(0.0f, 1.0f, 0.0f);
      if (mInvisTime > 0.1 && mInvisTime < 0.2 || mInvisTime > 0.3 && mInvisTime < 0.4 ||
            mInvisTime > 0.5 && mInvisTime < 0.6 || mInvisTime > 0.7)
         return;
   }
   else
      glColor3f(0.0f, 0.0f, 1.0f);
   
//   drawBoundingBox();

   if (mLife < 0.0f)
      glColor3f(1.0, 0.0, 0.0);
   else if (mState & INVINCIBLE) {
      glColor3f(0.0f, 1.0f, 0.0f);
      if (mInvisTime > 0.1 && mInvisTime < 0.2 || mInvisTime > 0.3 && 
       mInvisTime < 0.4 || mInvisTime > 0.5 && mInvisTime < 0.6 || 
       mInvisTime > 0.7)
         return;
   }
   else
      glColor3f(0.0, 0.0, 1.0);

   glPushMatrix();
      glTranslatef(mPos[X], mPos[Y], mPos[Z]);
      glScalef(0.025f, 0.025f, 0.025f);
      glTranslatef(0.0f, 5.0f, 0.0f);
      glRotatef(90.0f, -1, 0, 0);
      glColor3f(1.0, 1.0, 1.0);

      switch(myModel->GetState())
      {
      case MODEL_IDLE:
         myModel->Animate(0, 39, 0.07);
         break;
      case MODEL_RUN:
         myModel->Animate(40, 46, 0.07f);
         break;
      case MODEL_JUMP:
         myModel->Animate(67, 73, 0.07f);
         break;
      case MODEL_CROUCH:
         myModel->Animate(136, 154, 0.07f);
         break;
      default:
         break;
      }
   glPopMatrix();
}

void Player::reinitialize() {
   mCoins = 0;
   mLife = 100.0f;
}

void Player::step(double dt, Scene *scene) {
   double dx = 0.0, dy = 0.0, dz = 0.0;

   if (mState & JUMPING) {
      mJumpTime += dt;
//      printf("Jump time is %f, dt is %f, height is %f\n", mJumpTime, dt, mPos[Y]);
   }

   if (mState & INVINCIBLE) {
      mInvisTime += dt;

      if (mInvisTime > 0.75f) {
         mState &= ~INVINCIBLE;
      }
   }

   dx += mVel[X] * dt;
   dy += mVel[Y] * dt * 2.0;
   dz += mVel[Z] * dt;

   if (dy < 0.0) {             // IF NEGATIVE DY, YOU'RE FALLING.
//      if (!(mState & FALLING))
//         printf("Max height was %f\n", mPos[Y]);

      mState |= FALLING;
   }

   this->collide(&scene->mObjects, &dx, &dy, &dz);
   if (scene->mPrevScene)
      this->collide(&scene->mPrevScene->mObjects, &dx, &dy, &dz);   
   if (scene->mNextScene)
      this->collide(&scene->mNextScene->mObjects, &dx, &dy, &dz);

   this->collide(&scene->mFoes, &dx, &dy, &dz);
   if (scene->mPrevScene)
      this->collide(&scene->mPrevScene->mFoes, &dx, &dy, &dz);   
   if (scene->mNextScene)
      this->collide(&scene->mNextScene->mFoes, &dx, &dy, &dz);


   mVel[Y] += (-9.8 * dt) + (mAcc[Y] * dt);

   mPos[X] += dx;
   mPos[Y] += dy;
   mPos[Z] += dz;

   /* If the player is below a certain height, the player automatically dies. */
   if (mLife > 0.0f && mPos[Y] < -2.0) {
      mLife = -1.0f;
   }

   mMin[X] = mPos[X] - 0.5;
   mMin[Y] = mPos[Y] - 0.5;
   mMin[Z] = mPos[Z] - 0.5;

   mMax[X] = mPos[X] + 0.5;
   mMax[Y] = mPos[Y] + 0.5;
   mMax[Z] = mPos[Z] + 0.5;
}

void Player::decrLife() {
   if (!(mState & INVINCIBLE)) {
      #if defined(WIN32)
      PlaySound("powerdown.wav", NULL, SND_FILENAME | SND_ASYNC);
      #endif

      mState |= INVINCIBLE;
      mInvisTime = 0.0f;
      mLife -= 5.0f;
   }
}

void Player::collide(std::vector<Character *> *foes, double *dx, double *dy, double *dz) {
   for (std::vector<Character *>::iterator itr = foes->begin(); itr != foes->end(); itr++) {
      (*itr)->collide(this, dx, dy, dz);
   }
}

void Player::collide(std::vector<GameObject *> *objs, double *dx, double *dy, double *dz) {
   for (std::vector<GameObject *>::iterator itr = objs->begin(); itr != objs->end(); itr++) {
      (*itr)->collide(this, dx, dy, dz);
   }
}