#include "Stage1.h"
#include "World.h"

#define EOSOFFSET 200.0f

std::vector<Stage *> World::mStages;
std::vector<Stage *>::iterator World::mCurStage, World::mNextStage;


// HORIZONTAL SCENES ARE ABOUT 10 x 6 in size, ground is at 0
Stage1::Stage1() {
   if (!ParticleSystem::setup_textures())
      printf("Error reading textures\n");

   mCamera = new Camera();
   mCurKey = mCurSpecial = 0;
   mStageFlags = 0;
   mMusic = new MusicBox("stage1.xm");
   mMusic->PlayMusic();

   initScenes();
   LoadTextures();
   gettimeofday(&mCurTime, NULL);
}

GLvoid Stage1::LoadTextures() {	
    mBG1 = (Image *) malloc(sizeof(Image));
    mBG2 = (Image *) malloc(sizeof(Image));

    if (!mBG1 || !mBG2) {
        printf("Error allocating space for images");
        exit(-1);
    }

    cout << "trying to load backgrounds" << endl;
    if (!(ImageLoad("bg1.bmp", mBG1) && ImageLoad("bg2.bmp", mBG2))) {
      cout << "Error opening backgrounds" << endl;
       exit(-1);
    }
        /*  2d texture, level of detail 0 (normal), 3 components (red, green, blue),            */
    /*  x size from image, y size from image,                                              */    
    /*  border 0 (normal), rgb color data, unsigned byte data, data  */ 
    glBindTexture(GL_TEXTURE_2D, STAGE1BG1);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 mBG1->sizeX, mBG1->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, mBG1->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/

    glBindTexture(GL_TEXTURE_2D, STAGE1BG2);
    glTexImage2D(GL_TEXTURE_2D, 0, 3,
                 mBG2->sizeX, mBG2->sizeY,
                 0, GL_RGB, GL_UNSIGNED_BYTE, mBG2->data);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); /*  cheap scaling when image bigger than texture */    
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); /*  cheap scaling when image smalled than texture*/
    //free( TextureImage );
}


/* BMP file loader */

/*
* getint and getshort are help functions to load the bitmap byte by byte
*/
unsigned int Stage1::getint(FILE *fp) {
  int c, c1, c2, c3;
  
  /*  get 4 bytes */ 
  c = getc(fp);  
  c1 = getc(fp);  
  c2 = getc(fp);  
  c3 = getc(fp);
  
  return ((unsigned int) c) +   
    (((unsigned int) c1) << 8) + 
    (((unsigned int) c2) << 16) +
    (((unsigned int) c3) << 24);
}

unsigned int Stage1::getshort(FILE *fp) {
  int c, c1;
  
  /* get 2 bytes*/
  c = getc(fp);  
  c1 = getc(fp);
  
  return ((unsigned int) c) + (((unsigned int) c1) << 8);
}

/*  quick and dirty bitmap loader...for 24 bit bitmaps with 1 plane only.  */

int Stage1::ImageLoad(char *filename, Image *image) {
  FILE *file;
  unsigned long size;                 /*  size of the image in bytes. */
  unsigned long i;                    /*  standard counter. */
  unsigned short int planes;          /*  number of planes in image (must be 1)  */
  unsigned short int bpp;             /*  number of bits per pixel (must be 24) */
  char temp;                          /*  used to convert bgr to rgb color. */
  
  /*  make sure the file is there. */
  if ((file = fopen(filename, "rb"))==NULL) {
    printf("File Not Found : %s\n",filename);
    return 0;
  }
  
  /*  seek through the bmp header, up to the width height: */
  fseek(file, 18, SEEK_CUR);
  
  /*  No 100% errorchecking anymore!!! */
  
  /*  read the width */    image->sizeX = getint (file);
  
  /*  read the height */ 
  image->sizeY = getint (file);
  
  /*  calculate the size (assuming 24 bits or 3 bytes per pixel). */
  size = image->sizeX * image->sizeY * 3;
  
  /*  read the planes */    
  planes = getshort(file);
  if (planes != 1) {
    printf("Planes from %s is not 1: %u\n", filename, planes);
    return 0;
  }
  
  /*  read the bpp */    
  bpp = getshort(file);
  if (bpp != 24) {
    printf("Bpp from %s is not 24: %u\n", filename, bpp);
    return 0;
  }
  
  /*  seek past the rest of the bitmap header. */
  fseek(file, 24, SEEK_CUR);
  
  /*  read the data.  */
  image->data = (char *) malloc(size);
  if (image->data == NULL) {
    printf("Error allocating memory for color-corrected image data");
    return 0;	
  }
  
  if ((i = fread(image->data, size, 1, file)) != 1) {
    printf("Error reading image data from %s.\n", filename);
    return 0;
  }
  
  for (i=0;i<size;i+=3) { /*  reverse all of the colors. (bgr -> rgb) */
    temp = image->data[i];
    image->data[i] = image->data[i+2];
    image->data[i+2] = temp;
  }
  
  fclose(file); /* Close the file and release the filedes */
  
  /*  we're done. */
  return 1;
}

//--------------------------------------------------------------------


void Stage1::initScenes() {
   float curOffset = 0.0f;

   // SCENE 0
   mFirstScene = mCurScene = new Scene(curOffset);

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));
                                    
   mCurScene->addObject(new Ground( 1.0f + curOffset,  1.5f, -1.0f,
                                    4.8f + curOffset,  2.0f,  1.0f));

   mCurScene->addObject(new Coin(4.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(3.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(2.5f + curOffset, 2.5f, 0.0f));

   // SCENE 1
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;
   
   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    0.0f + curOffset,  0.0f,  1.0f));
                                    
   mCurScene->addObject(new Ground( 3.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));
                                    
   mCurScene->addObject(new Ground(-3.8f + curOffset,  3.0f, -1.0f,
                                   -1.0f + curOffset,  3.5f,  1.0f));
                                   
                                   
   mCurScene->addObject(new Coin(-3.5f + curOffset, 3.75f, 0.0f));
   mCurScene->addObject(new Coin(-2.5f + curOffset, 3.75f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 3.75f, 0.0f));

   mCurScene->addCharacter(new WalkingEnemy(0.0f + curOffset, 1.0, 0.0));

   // SCENE 2
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));
                                    
   mCurScene->addObject(new ParticleSystem(-3.0f + curOffset, 0.0f, -0.1f,
                                           -1.0f + curOffset, 0.25f,  0.1f, 1500));
                                           
   mCurScene->addObject(new Coin(-2.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 2.5f, 0.0f));

   mCurScene->addCharacter(new WalkingEnemy(5.0f + curOffset, 1.0f, 0.0f));

   // SCENE 3
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                   -3.0f + curOffset,  0.0f,  1.0f)); 
                                   
   mCurScene->addObject(new Ground(-1.8f + curOffset,  1.0f, -1.0f,
                                    3.8f + curOffset,  1.5f,  1.0f));
                                    
   mCurScene->addObject(new Coin(0.5f + curOffset, 2.0f, 0.0f));
   mCurScene->addObject(new Coin(1.5f + curOffset, 2.0f, 0.0f));

   // SCENE 4
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));
                                    
   mCurScene->addObject(new Ground(-3.0f + curOffset,  1.5f, -1.0f,
                                   -1.0f + curOffset,  2.0f,  1.0f));

   mCurScene->addObject(new Ground( 1.0f + curOffset,  1.5f, -1.0f,
                                    3.0f + curOffset,  2.0f,  1.0f));
                                    
   mCurScene->addObject(new ParticleSystem(-0.5f + curOffset, 0.0f, -0.1f,
                                            0.5f + curOffset, 0.5f,  0.1f, 1000));

   mCurScene->addObject(new Coin(-1.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin(-2.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin( 1.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin( 2.5f + curOffset, 0.5f, 0.0f));
                                            
   mCurScene->addCharacter(new PacingEnemy(2.5f + curOffset, 2.5f, 0.0f, 1.5));

   // SCENE 5
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;
   
   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));


   mCurScene->addObject(new FlowSystem(-0.5f + curOffset, 0.0f, -0.1f,
                                        0.5f + curOffset, 1.0f,  0.1f, 
                                        PARTICLETEX));
                                        
   mCurScene->addObject(new Coin(-0.5f + curOffset, 1.5f, 0.0));
   mCurScene->addObject(new Coin( 0.5f + curOffset, 1.5f, 0.0));

   mCurScene->addCharacter(new WalkingEnemy(0.0f + curOffset, 1.0f, 0.0f));
   mCurScene->addCharacter(new WalkingEnemy(5.0f + curOffset, 1.0f, 0.0f));

   // SCENE 6
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;
   
   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                   -3.0f + curOffset,  0.0f,  1.0f));
   
   mCurScene->addObject(new SwingingPlatform(-0.5f + curOffset, 0.5f, 0.0f,
                                             1.0f, 0.5f, 1.0f));

   mCurScene->addObject(new HorizontalPlatform(2.5f + curOffset, 0.5f, 0.0f,
                                               1.0f, 0.5f, 1.0f, 1.5f, X));

   mCurScene->addObject(new Coin(1.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(2.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(4.5f + curOffset, 2.5f, 0.0f));

   // SCENE 7
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new HorizontalPlatform(-4.0f + curOffset, 0.5f, 0.0f,
                                                1.0f, 0.5f, 1.0f, 1.0, X));

   mCurScene->addObject(new SwingingPlatform(0.5f + curOffset, 0.5f, 0.0f,
                                             1.0f, 0.5f, 1.0f));

   mCurScene->addObject(new Coin(-4.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(-2.5f + curOffset, 2.5f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 2.5f, 0.0f));


   // SCENE 8
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new Inset(-2.0f + curOffset, 0.0f, 0.0f + curOffset, 1.5f));


   mCurScene->addCharacter(new Boulder(6.0f + curOffset, 6.0f, 0.0f));

   // SCENE 9
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new Ground(-3.0f + curOffset,  1.5f, -1.0f,
                                    3.0f + curOffset, 2.0f,  1.0f));

   mCurScene->addObject(new FlowSystem(-0.5f + curOffset, 0.0f, -0.1f,
                                        0.5f + curOffset, 1.0f,  0.1f, PARTICLETEX));

   mCurScene->addObject(new ParticleSystem(-3.0f + curOffset, 2.0f, -0.1f,
                                           -2.0f + curOffset, 2.5f,  0.1f, 1000));
   
   mCurScene->addObject(new ParticleSystem(-0.25f + curOffset, 2.0f, -0.1f,
                                            0.25f + curOffset, 3.0f,  0.1f, 1000));

   mCurScene->addObject(new ParticleSystem( 2.0f + curOffset, 2.0f, -0.1f,
                                            3.0f + curOffset, 2.5f,  0.1f, 1000));


   mCurScene->addObject(new Coin(-2.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 0.5f, 0.0f));

   mCurScene->addObject(new Coin( 1.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin( 2.5f + curOffset, 0.5f, 0.0f));

   // SCENE 10
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                   -4.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new HorizontalPlatform(1.0f + curOffset, 0.5f, 0.0f,
                                               2.0f, 0.5f, 0.5f, 2.0f, X));

   mCurScene->addObject(new Coin(3.5f + curOffset, 1.5f, 0.0f));
   mCurScene->addObject(new Coin(4.5f + curOffset, 1.5f, 0.0f));

   // SCENE 11
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground( 4.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new HorizontalPlatform(-1.0f + curOffset, 0.5f, 0.0f,
                                               2.0f, 0.5f, 0.5f, 2.0f, X));

   mCurScene->addObject(new Coin(-0.5f + curOffset, 1.5f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 1.5f, 0.0f));

   // SCENE 12
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));


   mCurScene->addObject(new Ground(-3.0f + curOffset, 1.5f, -1.0f,
                                   -1.0f + curOffset, 2.0f,  1.0f));

   mCurScene->addObject(new Ground( 1.0f + curOffset, 1.5f, -1.0f,
                                    3.0f + curOffset, 2.0f,  1.0f));

   mCurScene->addObject(new ParticleSystem(-3.0f + curOffset, 0.0f, -0.1f,
                                           -2.0f + curOffset, 0.25f,  0.1f, 1000));

   mCurScene->addObject(new ParticleSystem(-0.25f + curOffset, 0.0f, -0.1f,
                                            0.25f + curOffset, 0.5,  0.1f, 1000));

   mCurScene->addObject(new ParticleSystem( 2.0f + curOffset, 0.0f, -0.1f,
                                            3.0f + curOffset, 0.25,  0.1f, 1000));

   mCurScene->addObject(new Coin(-2.5f + curOffset, 3.0f, 0.0f));
   mCurScene->addObject(new Coin(-1.5f + curOffset, 3.0f, 0.0f));
   mCurScene->addObject(new Coin( 1.5f + curOffset, 3.0f, 0.0f));
   mCurScene->addObject(new Coin( 2.5f + curOffset, 3.0f, 0.0f));

   mCurScene->addCharacter(new PacingEnemy(2.5f + curOffset, 2.5f, 0.0f, 1.0f));

   // SCENE 13
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-2.5f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new Ground(-1.0f + curOffset, 1.5f, -1.0f,
                                    1.0f + curOffset, 2.0f,  1.0f));

   mCurScene->addObject(new Ground( 1.5f + curOffset, 2.5f, -1.0f,
                                    3.5f + curOffset, 3.0f,  1.0f));
   
   mCurScene->addCharacter(new Boulder(5.0f + curOffset, 5.0f, 0.0f));

   // SCENE 14
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(1.0f + curOffset, 2.5f, -1.0f,
                                   2.0f + curOffset, 3.0f,  1.0f));

   mCurScene->addObject(new Ground(1.0f + curOffset, 0.0f, -1.0f,
                                   2.0f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new HorizontalPlatform(-2.5f + curOffset, 1.5f, 0.0f,
                                               1.0f, 0.5f, 1.0f, 1.5f, Y));

   mCurScene->addObject(new Coin(-2.5f + curOffset, 3.5f, 0.0f));
   mCurScene->addObject(new Coin( 1.5f + curOffset, 3.5f, 0.0f));

   mCurScene->addCharacter(new FloatingEnemy(-4.5f + curOffset, 2.5f, 0.0f, 1.5f));
   mCurScene->addCharacter(new FloatingEnemy( 4.5f + curOffset, 2.5f, 0.0f, 1.5f));

   // SCENE 15
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-2.0f + curOffset, -0.5f, -1.0f,
                                   -1.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new Ground( 1.0f + curOffset, 0.5f, -1.0f,
                                    2.0f + curOffset, 1.0f,  1.0f));

   mCurScene->addObject(new Ground( 4.0f + curOffset, 0.0f, -1.0f,
                                    5.0f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new Ground( 4.0f + curOffset, 3.5f, -1.0f,
                                    5.0f + curOffset, 4.0f,  1.0f));

   mCurScene->addCharacter(new FloatingEnemy(-1.0f + curOffset, 3.5f, 0.0f, 1.0f));
   mCurScene->addCharacter(new FloatingEnemy( 0.0f + curOffset, 4.0f, 0.0f, 1.0f));
   mCurScene->addCharacter(new FloatingEnemy( 1.0f + curOffset, 4.5f, 0.0f, 1.0f));
   mCurScene->addCharacter(new FloatingEnemy( 3.5f + curOffset, 2.0f, 0.0f, 1.5f));
   
   // SCENE 16
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-3.0f + curOffset, 1.0f, -1.0f,
                                   -2.0f + curOffset, 1.5f,  1.0f));

   mCurScene->addObject(new Ground( 0.0f + curOffset, 0.0f, -1.0f,
                                    1.0f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new Ground( 0.0f + curOffset, 3.0f, -1.0f,
                                    1.0f + curOffset, 3.5f,  1.0f));

   mCurScene->addObject(new FlowSystem(-3.0f + curOffset, 1.5f, -0.1f,
                                       -2.0f + curOffset, 2.5f,  0.1f, 
                                        PARTICLETEX));

   mCurScene->addObject(new ParticleSystem( 0.0f + curOffset, 0.5f, -0.1f,
                                            1.0f + curOffset, 0.75f, 0.1f, 1000));

   mCurScene->addObject(new SwingingPlatform(4.5f + curOffset, 1.0f, 0.0f,
                                             1.0f, 0.5f, 1.0f));


   // SCENE 17
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(4.0f + curOffset, -1.0f, -1.0f,
                                   5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new HorizontalPlatform(-0.5f + curOffset, 1.0f, 0.0f,
                                                1.0f, 0.5f, 1.0f, 2.0f, Y));

   mCurScene->addCharacter(new FloatingEnemy(-1.5f + curOffset, 2.0f, 0.0f, 1.5f));

   // SCENE 18
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new FallingPlatform(-1.75f + curOffset, 0.0f, -1.0f,
                                            -1.25f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new FallingPlatform( 0.25f + curOffset, 0.0f, -1.0f,
                                             0.75f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new FallingPlatform( 2.25f + curOffset, 0.0f, -1.0f,
                                             2.75f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new FallingPlatform( 4.25f + curOffset, 0.0f, -1.0f,
                                             4.75f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new Coin(-1.5f + curOffset, 1.0f, 0.0f));
   mCurScene->addObject(new Coin( 0.5f + curOffset, 1.0f, 0.0f));
   mCurScene->addObject(new Coin( 2.5f + curOffset, 1.0f, 0.0f));
   mCurScene->addObject(new Coin( 4.5f + curOffset, 1.0f, 0.0f));

   mCurScene->addCharacter(new FloatingEnemy(-3.0f + curOffset, 1.0f, 0.0f, 1.5f));

   // SCENE 19
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;
   
   mCurScene->addObject(new FallingPlatform(-3.75f + curOffset, 0.0f, -1.0f,
                                            -3.25f + curOffset, 0.5f,  1.0f));

   mCurScene->addObject(new Ground(-1.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));

   mCurScene->addObject(new Coin(-3.5f + curOffset, 1.0f, 0.0f));
   mCurScene->addObject(new Coin(0.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin(1.5f + curOffset, 0.5f, 0.0f));
   mCurScene->addObject(new Coin(2.5f + curOffset, 0.5f, 0.0f));

   mCurScene->addCharacter(new FloatingEnemy(-2.0f + curOffset, 1.0f, 0.0f, 1.5f));

   // SCENE 20, PLACEHOLDER FOR END OF STAGE
   curOffset += SCENE_WIDTH;
   mCurScene->mNextScene = new Scene(curOffset);
   mCurScene->mNextScene->mPrevScene = mCurScene;
   mCurScene = mCurScene->mNextScene;

   mCurScene->addObject(new Ground(-5.0f + curOffset, -1.0f, -1.0f,
                                    5.0f + curOffset,  0.0f,  1.0f));


// RESET CURSCENE TO FIRST
   mCurScene = mFirstScene;
}

Stage1::~Stage1() {

}

void Stage1::display() {
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   frameStart();
   glMatrixMode(GL_MODELVIEW);

   glPushMatrix();
      glLoadIdentity();

//      gluLookAt(mCamera->mCameraX, mCamera->mCameraY, mCamera->mCameraZ,
//         mCamera->mLookAtX, mCamera->mLookAtY, mCamera->mLookAtZ,
//         mCamera->mUpX, mCamera->mUpY, mCamera->mUpZ);

      if (!(Stage::mPlayer.mState & ORTHO)) {
         if (Stage::mPlayer.mState & CAMERABEHIND)
            gluLookAt(Stage::mPlayer.mPos[X] - 4.0, mCamera->mCameraY, Stage::mPlayer.mPos[Z],
                     Stage::mPlayer.mPos[X],       mCamera->mLookAtY, Stage::mPlayer.mPos[Z],
                     mCamera->mUpX,          mCamera->mUpY,     mCamera->mUpZ);

         else
            gluLookAt(Stage::mPlayer.mPos[X],       mCamera->mCameraY, mCamera->mCameraZ,
                     Stage::mPlayer.mPos[X],        mCamera->mLookAtY, mCamera->mLookAtZ,
                     mCamera->mUpX,          mCamera->mUpY,     mCamera->mUpZ);
      }
      else if (Stage::mPlayer.mState & ORTHO) {
         printf("Ortho camera\n");
         gluLookAt(Stage::mPlayer.mPos[X], 0, 5,
                   Stage::mPlayer.mPos[X], 0, 0,
                   0, 1, 0);
      }

      glPushMatrix();
//      glTranslatef(Stage::mPlayer.mPos[X] - ((Stage::mPlayer.mPos[X] + 5.0f) * 0.2f), 0.0f, -10.0f);      
      glTranslatef(Stage::mPlayer.mPos[X] * 0.8f + 1.0f, 0.0f, -10.0f);
         glColor3f(1.0f, 1.0f, 1.0f);
      
         glEnable(GL_TEXTURE_2D);
         glBindTexture(GL_TEXTURE_2D, STAGE1BG1);
         glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 0.0f); glVertex3f(-20.0f, -20.0f, 0.0f);
            glTexCoord2f(0.0f, 1.0f); glVertex3f(-20.0f,  20.0f, 0.0f);
            glTexCoord2f(1.0f, 1.0f); glVertex3f( 20.0f,  20.0f, 0.0f);
            glTexCoord2f(1.0f, 0.0f); glVertex3f( 20.0f, -20.0f, 0.0f);
         glEnd();

         glBindTexture(GL_TEXTURE_2D, STAGE1BG2);
         glBegin(GL_QUADS);
            glTexCoord2f(0.0f, 0.0f); glVertex3f( 20.0f, -20.0f, 0.0f);
            glTexCoord2f(0.0f, 1.0f); glVertex3f( 20.0f,  20.0f, 0.0f);
            glTexCoord2f(1.0f, 1.0f); glVertex3f( 60.0f,  20.0f, 0.0f);
            glTexCoord2f(1.0f, 0.0f); glVertex3f( 60.0f, -20.0f, 0.0f);
         glEnd();
         glDisable(GL_TEXTURE_2D);
      glPopMatrix();

      glDisable(GL_LIGHTING);
      if (mCurScene->mPrevScene)
         mCurScene->mPrevScene->draw();
      if (mCurScene->mNextScene)
         mCurScene->mNextScene->draw();
      mCurScene->draw();
      Stage::mPlayer.draw();

   glPopMatrix();
   frameEnd(GLUT_BITMAP_HELVETICA_12, 1.0f, 0.0f, 0.0f, 0, 0.95);
   glutSwapBuffers();
}

void Stage1::idle() {
   mPrevTime.tv_sec = mCurTime.tv_sec;
   mPrevTime.tv_usec = mCurTime.tv_usec;
   gettimeofday(&mCurTime, NULL);
   double dt = PASSED;

   if (Stage::mPlayer.mLife < 0.0f && mStageFlags & IMMORTAL) { // DIED ON GOD MODE
      Stage::mPlayer.mLife = 100.0f;
      Stage::mPlayer.mPos[Y] = 10.0;
      Stage::mPlayer.mMin[Y] = 9.5;
      Stage::mPlayer.mMax[Y] = 10.5;
      Stage::mPlayer.mVel[Y] = 0.0f;
   }
   else if (Stage::mPlayer.mLife < 0.0f) {                     // PLAYER HAS DIED
      if (Stage::mPlayer.mLives == 0) {                        // NO LIVES REMAINING,
         exit(0);                                        // GAME OVER
      }
      else {                                             // ELSE, RESTART
         Stage::mPlayer.mLives--;
         restartStage();
      }
   }
   else if (mCurScene->mOffset == EOSOFFSET) {      // STAGE COMPLETE!
      printf("Stage complete!\n");
      Stage::mPlayer.mPos[X] = 0.0f;
      Stage::mPlayer.mPos[Y] = 5.0f;
      Stage::mPlayer.mPos[Z] = 0.0f;

      Stage::mPlayer.mMin[X] = -0.5f;
      Stage::mPlayer.mMin[Y] = 4.5f;
      Stage::mPlayer.mMin[Z] = -0.5f;

      Stage::mPlayer.mMax[X] = 0.5f;
      Stage::mPlayer.mMax[Y] = 5.5f;
      Stage::mPlayer.mMax[Z] = 0.5f;
      delete mMusic;
      mStageFlags = STAGECOMPLETE;
      World::mCurStage++;
      (*World::mCurStage)->initStage();
   }
   else {
      Coin::gstep(dt);

      if (Stage::mPlayer.mState & SPACEPRESSED)
         Stage::mPlayer.jump();

      Stage::mPlayer.mVel[X] = 0.0;

      switch(mCurSpecial) {
         case GLUT_KEY_LEFT :                    // LEFT ARROW MOVES PLAYER LEFT
            Stage::mPlayer.mVel[X] = -5.0f;
            glutPostRedisplay();
            break;
         case GLUT_KEY_RIGHT :                   // RIGHT ARROW MOVES PLAYER RIGHT
            Stage::mPlayer.mVel[X] = 5.0f;
            glutPostRedisplay();
            break;
         default :
            break;
      }

      Stage::mPlayer.step(dt, mCurScene);
      mCurScene->step(dt);

      if (mCurScene->mPrevScene)
         mCurScene->mPrevScene->step(dt);
      if (mCurScene->mNextScene)
         mCurScene->mNextScene->step(dt);

      if (mCurScene->mNextScene && Stage::mPlayer.mPos[X] >= mCurScene->mNextScene->mOffset - 5.0f)
         mCurScene = mCurScene->mNextScene;
      if (mCurScene->mPrevScene && Stage::mPlayer.mPos[X] < mCurScene->mOffset - 5.0f)
         mCurScene = mCurScene->mPrevScene;
   }

   glutPostRedisplay();
}

void Stage1::keyboard(unsigned char key, int x, int y) {
   mCurKey = key;

   switch (key) {

      case ' ' :
         Stage::mPlayer.mState |= SPACEPRESSED;
         break;
      case 'c' : case 'C' :                     // TOGGLE THE CAMERA
         Stage::mPlayer.mState ^= CAMERABEHIND;
         break;
      case 'g' : case 'G' :                     // TOGGLE IMMORTAL MODE
         printf("IMMORTAL MODE\n");
         mStageFlags ^= IMMORTAL;
         break;
      case 'o' : case 'O' :                     // TOGGLE ORTHOGRAPHIC PROJECTION --DISABLED--
         Stage::mPlayer.mState ^= ORTHO;
         printf("Orthographic Projection is %d\n", Stage::mPlayer.mState & ORTHO);
         glutPostRedisplay();
         break;
      case 'q' : case 'Q' :                     // QUIT THE GAME
         exit(0);
         break;
      case 'r' : case 'R' :                     // RESET THE GAME
         restartStage();
         break;
      case 'z' : case 'Z' :                     // SKIP STAGE
         Stage::mPlayer.mPos[X] = 0.0f;
         Stage::mPlayer.mPos[Y] = 5.0f;
         Stage::mPlayer.mPos[Z] = 0.0f;

         Stage::mPlayer.mMin[X] = -0.5f;
         Stage::mPlayer.mMin[Y] = 4.5f;
         Stage::mPlayer.mMin[Z] = -0.5f;

         Stage::mPlayer.mMax[X] = 0.5f;
         Stage::mPlayer.mMax[Y] = 5.5f;
         Stage::mPlayer.mMax[Z] = 0.5f;
         mStageFlags = STAGECOMPLETE;
         break;
      default :
         break;
   }
}

void Stage1::keyboardUp(unsigned char key, int x, int y) {
   if (key == ' ') {
      Stage::mPlayer.mState &= ~SPACEPRESSED;
      Stage::mPlayer.stopJump();
   }

   mCurKey = 0;
}

void Stage1::reshape(int width, int height) {
   glMatrixMode(GL_PROJECTION);
      glLoadIdentity();

      if (Stage::mPlayer.mState & ORTHO) {
         glOrtho(-5.0, 6.0, -1.0, 5.0, -10, 50);
      }
      else {
         gluPerspective(45, (double)width / height, .1, 50);
      }

   glMatrixMode(GL_MODELVIEW);
      glViewport(0, 0, width, height);
  
   glutPostRedisplay();
}

void Stage1::restartStage() {
   #if defined(WIN32)
   PlaySound("death.wav", NULL, SND_FILENAME | SND_ASYNC);
   #endif

   for (Scene *curScene = mFirstScene; curScene; curScene = curScene->mNextScene) {
      for (std::vector<GameObject *>::iterator itr = curScene->mObjects.begin(); itr != curScene->mObjects.end(); itr++) {
         (*itr)->reinitialize();
      }
      for (std::vector<Character *>::iterator itr = curScene->mFoes.begin(); itr != curScene->mFoes.end(); itr++) {
         (*itr)->reinitialize();
      }
   }

   Stage::mPlayer.reinitialize();

   Stage::mPlayer.mPos[X] = 0.0;
   Stage::mPlayer.mPos[Y] = 10.0;
   Stage::mPlayer.mPos[Z] = 0.0;

   Stage::mPlayer.mMin[X] = -0.5;
   Stage::mPlayer.mMin[Y] = 9.5;
   Stage::mPlayer.mMin[Z] = -0.5;

   Stage::mPlayer.mMax[X] = 0.5;
   Stage::mPlayer.mMax[Y] = 10.5;
   Stage::mPlayer.mMax[Z] = 0.5;
   mCurScene = mFirstScene;
}

void Stage1::special(int key, int x, int y) {
   if (key == GLUT_KEY_UP)
      Stage::mPlayer.mState |= UPPRESSED;
   else
   {
      if(Stage::mPlayer.myModel->GetState() != MODEL_JUMP)
         Stage::mPlayer.myModel->SetState(MODEL_RUN);
      mCurSpecial = key;
   }
}

void Stage1::specialUp(int key, int x, int y) {
   if (key == GLUT_KEY_UP)
      Stage::mPlayer.mState &= ~UPPRESSED;
   else
   {
      if(key != GLUT_KEY_DOWN)
      {
         if(Stage::mPlayer.myModel->GetState() != MODEL_JUMP)
            Stage::mPlayer.myModel->SetState(MODEL_IDLE);
      }
      mCurSpecial = 0;
   }
}

void Stage1::frameEnd(void *font, GLclampf r, GLclampf g, GLclampf b, GLfloat x, GLfloat y) {
  /* font: font to use, e.g., GLUT_BITMAP_HELVETICA_10
     r, g, b: text colour
     x, y: text position in window: range [0,0] (bottom left of window)
           to [1,1] (top right of window). */

   float elapsedTime; 
   char str[40]; 
   char *ch; 
   GLint matrixMode;
   GLboolean lightingOn;

   gettimeofday(&frameEndTime, NULL);
   elapsedTime = ELAPSED;

   lightingOn = glIsEnabled(GL_LIGHTING);        /* lighting on? */

   if (lightingOn)
      glDisable(GL_LIGHTING);

   glGetIntegerv(GL_MATRIX_MODE, &matrixMode);  /* matrix mode? */

   glMatrixMode(GL_PROJECTION);
   glPushMatrix();
      glLoadIdentity();
      gluOrtho2D(0.0, 1.0, 0.0, 1.0);
      glMatrixMode(GL_MODELVIEW);

      glPushMatrix();
         glLoadIdentity();   
         glPushAttrib(GL_COLOR_BUFFER_BIT);       /* save current colour */
         glColor3f(r, g, b);

         sprintf(str, "Frames per second: %2.0f", 1.0 / elapsedTime);
         glRasterPos3f(x, y, 0.0);

         for(ch = str; *ch; ch++) {
            glutBitmapCharacter(font, (int)*ch);
         }

         sprintf(str, "Life: %2.0f", Stage::mPlayer.getLife());
         glRasterPos3f(x, y - 0.05, 0.0);

         for(ch = str; *ch; ch++) {
            glutBitmapCharacter(font, (int)*ch);
         }

//         sprintf(str, "Current offset is %f", mCurScene->mOffset);
         sprintf(str, "Lives: %d\n", Stage::mPlayer.mLives);
         glRasterPos3f(x, y - 0.10f, 0.0);

         for(ch = str; *ch; ch++) {
            glutBitmapCharacter(font, (int)*ch);
         }

         sprintf(str, "Coins: %d", Stage::mPlayer.mCoins);
         glRasterPos3f(x, y - 0.15f, 0.0);

         for(ch = str; *ch; ch++) {
            glutBitmapCharacter(font, (int)*ch);
         }

         glPopAttrib();
      glPopMatrix();
      glMatrixMode(GL_PROJECTION);
   glPopMatrix();
   glMatrixMode(matrixMode);

   if (lightingOn)
      glEnable(GL_LIGHTING);
}
