#include "WalkingEnemy.h"

WalkingEnemy::WalkingEnemy(float posx, float posy, float posz) {
   mPos[X] = mStartPos[X] = posx;
   mPos[Y] = mStartPos[Y] = posy;
   mPos[Z] = mStartPos[Z] = posz;

   mMin[X] = mPos[X] - 0.5f;
   mMin[Y] = mPos[Y] - 0.5f;
   mMin[Z] = mPos[Z] - 0.5f;

   mMax[X] = mPos[X] + 0.5f;
   mMax[Y] = mPos[Y] + 0.5f;
   mMax[Z] = mPos[Z] + 0.5f;

   mState = 0;

   mVel[X] = -2.0f;
   mVel[Y] = mVel[Z] = 0.0f;

   myModel = new CMD2Model;
   myModel->Load("models\\ogro\\tris.md2", "models\\ogro\\ogrobase.pcx");
   //myModel->Load("MARVIN\\tris.md2", "MARVIN\\Marvin.pcx");
   //myModel->Load("OGRO\\tris.MD2", "OGRO\\hobgoblin.bmp");
   modelState = MODEL_RUN;
}

WalkingEnemy::~WalkingEnemy() {

}

void WalkingEnemy::draw() {
   if (mEnabled) {
      glPushMatrix();
         glColor3f(1.0, 0.0, 0.0);
/*
drawBoundingBox();

         glBegin(GL_LINES);
            glVertex3f(mMin[X], mMin[Y], mMin[Z]);
            glVertex3f(mMin[X], mMin[Y], mMax[Z]);
            
            glVertex3f(mMin[X], mMin[Y], mMax[Z]);
            glVertex3f(mMax[X], mMin[Y], mMax[Z]);

            glVertex3f(mMax[X], mMin[Y], mMax[Z]);
            glVertex3f(mMax[X], mMin[Y], mMin[Z]);

            glVertex3f(mMax[X], mMin[Y], mMin[Z]);
            glVertex3f(mMin[X], mMin[Y], mMin[Z]);

            glVertex3f(mMin[X], mMin[Y], mMin[Z]);
            glVertex3f((mMax[X] + mMin[X]) * 0.5f, mMax[Y], (mMax[Z] + mMin[Z]) * 0.5f);

            glVertex3f(mMin[X], mMin[Y], mMax[Z]);
            glVertex3f((mMax[X] + mMin[X]) * 0.5f, mMax[Y], (mMax[Z] + mMin[Z]) * 0.5f);

            glVertex3f(mMax[X], mMin[Y], mMin[Z]);
            glVertex3f((mMax[X] + mMin[X]) * 0.5f, mMax[Y], (mMax[Z] + mMin[Z]) * 0.5f);

            glVertex3f(mMax[X], mMin[Y], mMax[Z]);
            glVertex3f((mMax[X] + mMin[X]) * 0.5f, mMax[Y], (mMax[Z] + mMin[Z]) * 0.5f);
         glEnd();
*/
         glTranslatef(mPos[X], mPos[Y], mPos[Z]);
         glScalef(0.025f, 0.025f, 0.025f);
         glTranslatef(0.0f, 5.0f, 0.0f);
         glRotatef(180.0f, 0.f, 1.0, 0.0);
         glRotatef(90.0f, -1, 0, 0);
         glColor3f(1.0, 1.0, 1.0);

         myModel->Animate(40, 46, 0.07f);
      glPopMatrix();
   }
}

void WalkingEnemy::reinitialize() {
   mEnabled = true;
   mPos[X] = mStartPos[X];
   mPos[Y] = mStartPos[Y];
   mPos[Z] = mStartPos[Z];

   mMin[X] = mPos[X] - 0.5f;
   mMin[Y] = mPos[Y] - 0.5f;
   mMin[Z] = mPos[Z] - 0.5f;

   mMax[X] = mPos[X] + 0.5f;
   mMax[Y] = mPos[Y] + 0.5f;
   mMax[Z] = mPos[Z] + 0.5f;

   mVel[X] = -2.0f;
   mVel[Y] = mVel[Z] = 0.0f;
}

void WalkingEnemy::step(double dt, Scene *scene) {
   if (mEnabled) {
      double dx = mVel[X] * dt;
      double dy = mVel[Y] * dt;
      double dz = mVel[Z] * dt;

      this->collide(&scene->mObjects, &dx, &dy, &dz);
      if (scene->mPrevScene)
         this->collide(&scene->mPrevScene->mObjects, &dx, &dy, &dz);   
      if (scene->mNextScene)
         this->collide(&scene->mNextScene->mObjects, &dx, &dy, &dz);

      mVel[Y] += -9.8 * dt;

      mPos[X] += mVel[X] * dt;
      mPos[Y] += dy;

      mEnabled = mPos[Y] > -1.0f;

      mMin[X] = mPos[X] - 0.5;
      mMin[Y] = mPos[Y] - 0.5;
      mMin[Z] = mPos[Z] - 0.5;

      mMax[X] = mPos[X] + 0.5;
      mMax[Y] = mPos[Y] + 0.5;
      mMax[Z] = mPos[Z] + 0.5;
   }
}

void WalkingEnemy::collide(Player *player, double *dx, double *dy, double *dz) {
   if (mEnabled && collideAxis(player, X) && collideAxis(player, Y) && 
    collideAxis(player, Z)) {
      if (player->mPos[Y] >= mMax[Y]) {                                
         player->mState &= ~JUMPING;                                           
         player->forceJump();
         mEnabled = false;

         if (!(player->mState & SPACEPRESSED))
            player->stopJump();
      }             
      else {
         player->decrLife();
      }
   }
}

void WalkingEnemy::collide(std::vector<GameObject *> *objs, double *dx, double *dy, double *dz) {
   for (std::vector<GameObject *>::iterator itr = objs->begin(); itr != objs->end(); itr++) {
      (*itr)->collide(this, dx, dy, dz);
   }
}