//#include "Stage.h"
#include "Stage1.h"
#include "Stage2.h"
#include "World.h"

#if defined(WIN32)
   #include <GL/glut.h>
#else
   #include <GLUT/glut.h>
#endif

#define WINWIDTH  300
#define WINHEIGHT 300

//Stage *stage1;
World world;

void display()                                   {world.display();}
void idle()                                      {world.idle();}
void keyboard(unsigned char key, int x, int y)   {world.keyboard(key, x, y);}
void keyboardUp(unsigned char key, int x, int y) {world.keyboardUp(key, x, y);}
void reshape(int x, int y)                       {world.reshape(x, y);}
void special(int key, int x, int y)              {world.special(key, x, y);}
void specialUp(int key, int x, int y)            {world.specialUp(key, x, y);}

void winInit() {
   glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
   glutInitWindowSize(WINWIDTH, WINHEIGHT); 
   glutInitWindowPosition(100, 100);
   glutCreateWindow("CSC 476 Final Project");
   glClearColor(0.0, 0.0, 0.0, 1.0);

//   stage1 = new Stage1();

   glEnable(GL_DEPTH_TEST);
   glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(60, (double)WINWIDTH / WINHEIGHT, .1, 10);

   glMatrixMode(GL_MODELVIEW);
      glViewport(0, 0, WINWIDTH, WINHEIGHT); //image to pixel

   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

   GLfloat light_pos[4] = {1.0, 1.0, 1.5, 1.0};
   GLfloat light_amb[4] = {0.6, 0.6, 0.6, 1.0};
   GLfloat light_diff[4] = {0.6, 0.6, 0.6, 1.0};
   GLfloat light_spec[4] = {0.8, 0.8, 0.8, 1.0};

   glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
   glLightfv(GL_LIGHT0, GL_AMBIENT, light_amb);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diff);
   glLightfv(GL_LIGHT0, GL_SPECULAR, light_spec);

   GLfloat mAmbient[4] = {0.5, 0.5, 1.0, 1.0};
   GLfloat mDiffuse[4] = {1.0, 1.0, 1.0, 1.0};
   GLfloat mSpecular[4] = {0.2, 0.2, 0.9, 1.0};
   GLfloat mShininess[1] = {25.0};

   glMaterialfv(GL_FRONT, GL_AMBIENT,   mAmbient);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   mDiffuse);
   glMaterialfv(GL_FRONT, GL_SPECULAR,  mSpecular);
   glMaterialfv(GL_FRONT, GL_SHININESS, mShininess);   
}

void initCallBacks() {
   glutDisplayFunc(display);
   glutIdleFunc(idle);
   glutKeyboardFunc(keyboard);
   glutKeyboardUpFunc(keyboardUp);
   glutReshapeFunc(reshape);
   glutSpecialFunc(special);
   glutSpecialUpFunc(specialUp);
}

void initTextures() {
   Cave::LoadTexture("stone.bmp", STONETEX);
   Ground::LoadTexture("wood.bmp", WOODTEX);
   Pit::LoadTexture("Fire.bmp", FIRETEX);
   CrushingWall::LoadTexture("carpet1abump.bmp", CARPETTEX);
   MovingSpike::LoadTexture("boulderBump2.bmp", MOVINGSPIKE);
   SwingingPlatform::LoadTexture("metal5.bmp", SWNGNGPLTFM);
   HorizontalPlatform::LoadTexture("weirdBlue.bmp", HORZNTPLFM);
   Boulder::LoadTexture("boulderBump.bmp", BOULDERTEX);
   FallingPlatform::LoadTexture("weirdBlue.bmp", FALLNGPLFTM);
   Inset::LoadTexture("darkerWood.bmp", INSETTEX);
}

int main(int argc, char **argv) {
   glutInit(&argc, argv);
   winInit();
   initCallBacks();
   initTextures();

   world.addStage(new Stage1());
   world.addStage(new Stage2());
   world.init();

   glutMainLoop();
   return 0;
}